/*
 * Copyright (c) 2003, Enrique Jose Garcia Cota
 * All rights reserved.
 *
 * This file is part of antlraux. antlraux is distributed under 
 * the BSD license. See the license terms in file license.txt.
 *
 */

package antlraux.context.asts;

import antlraux.context.types.Type;
import antlraux.util.LexInfoAST;
import antlr.collections.AST;

/**
 * This class may be used when modelling an AST with type information. It sould
 * not be used for expressions; {@link ExpressionAST} is more suited for that.
 * This class could be used for easily obtaining the
 * {@link antlraux.context.types.Type} of tree node that codes a type. Typically
 * you'll have that kind of node at the beginning of your declarations. For
 * example, in a languaje that declares variables like in C you may have:
 * 
 * <pre>
 * int a;
 * </pre>
 * 
 * The rule that recognizes this declaration could be:
 * 
 * <pre>
 *   declaration: type IDENT SEMI ;
 *   type : TYPE_INT
 *        | TYPE_FLOAT
 *        | ...
 *        ;
 * </pre>
 * 
 * {@link TypeAST} sould be used in the <tt>type</tt> rule, like this:
 * 
 * <pre>
 *   type : TYPE_INT &lt;AST=antlraux.contex.asts.TypeAST&gt;
 *          { ##.setExpType(getType(TYPE_INT)); }
 *        | TYPE_FLOAT
 *          ...
 *        ;
 * </pre>
 * 
 * Once this is done, you can obtain one type's "expression type" just calling
 * {@link #getExpType()}.
 * <p>
 * Warning: {@link TypeAST}, {@link ExpressionAST} and {@link ScopeAST} copy
 * first child and sibling with their only constructor. See their constructor's
 * comments for details.
 */
public class TypeAST extends LexInfoAST {
    Type expType = null;

    public TypeAST() {
        super();
    }

    /**
     * This constructors calls {@link #initialize(AST)} so it copies other's
     * first child and next sibling.
     */
    public TypeAST(AST other) {
        super();
        initialize(other);
    }

    public void setExpType(Type t) {
        expType = t;
    }

    public Type getExpType() {
        return expType;
    }

    /**
     * Warning: this implementation of initialize copies <tt>ast</tt>'s first
     * child and next sibling.
     */
    public void initialize(AST ast) {
        super.initialize(ast);
        setFirstChild(ast.getFirstChild());
        setNextSibling(ast.getNextSibling());

        if (ast instanceof TypeAST) {
            this.expType = ((TypeAST) ast).expType;
        }
    }

    protected static boolean verboseStringConversion = false;

    /**
     * If set to true, this adds some additional information to the string
     * generated by {@link #toString()} (shows the AST's type)
     */
    public static void setVerboseStringConversion(boolean b) {
        verboseStringConversion = b;
    }

    public String toString() {
        StringBuffer sb = new StringBuffer(super.toString());

        if (verboseStringConversion) {
            sb.append(" (type: ");
            sb.append(expType);
            sb.append(")");
        }
        return sb.toString();
    }

}